import round from "lodash.round";
import { sanitize_ohlc } from "./feature_ohlc";
export const match_pattern_features = (ohlc_dataset, ohlc_pattern) => {
    const calculated_pattern = [];
    if (ohlc_dataset.length) {
        // Sanitize Dataset
        const ohlc_data = sanitize_ohlc(ohlc_dataset);
        const initial_tick = ohlc_data[0];
        // Calculate Slicing from Data and Pattern
        const data_indexes = [];
        if (ohlc_pattern) {
            for (const [idx, tick] of ohlc_pattern.entries()) {
                const tick_percentage = round(tick.tick_distance, 2);
                const data_idx = round(tick_percentage * 0.01 * (ohlc_data.length - 1));
                data_indexes.push(data_idx);
            }
        }
        for (const [idx, tick] of ohlc_data.entries()) {
            const tick = ohlc_data[idx];
            const tick_body = Math.abs(tick.close - tick.open);
            const lower_boundary = tick.open < tick.close ? tick.open : tick.close;
            const upper_boundary = tick.open < tick.close ? tick.close : tick.open;
            if (idx === 0) {
                calculated_pattern.push({
                    close_diff_percent: 0,
                    mid_diff_percent: 0,
                    lower_wick_percent: round((100 * (lower_boundary - tick.low)) / tick_body, 2),
                    upper_wick_percent: round((100 * (tick.high - upper_boundary)) / tick_body, 2),
                    volume_diff_percent: 0,
                    is_highlighted: true,
                });
            }
            else {
                const prev_tick = ohlc_data[idx - 1];
                calculated_pattern.push({
                    close_diff_percent: round((100 * (tick.close - initial_tick.close)) / initial_tick.close, 2),
                    mid_diff_percent: round((100 * (tick.mid - initial_tick.mid)) / initial_tick.mid, 2),
                    lower_wick_percent: round((100 * (lower_boundary - tick.low)) / tick_body, 2),
                    upper_wick_percent: round((100 * (tick.high - upper_boundary)) / tick_body, 2),
                    volume_diff_percent: round((100 * (tick.volume - prev_tick.volume)) / prev_tick.volume, 2),
                    is_highlighted: data_indexes.includes(idx),
                });
            }
        }
        return calculated_pattern;
    }
    return calculated_pattern;
};
export const match_pattern_closeness = (ohlc_patt, ohlc_data, settings) => {
    // Sanitize Dataset
    const ohlc_pattern = sanitize_ohlc(ohlc_patt);
    // Dataset Validation
    if (ohlc_pattern.slice(1, ohlc_pattern.length).filter((tick) => !tick.tick_distance || tick.tick_distance <= 0).length) {
        return {
            closeness: 0,
            error: "tick_distance not set / incorrect",
        };
    }
    if (ohlc_data.length < ohlc_pattern.length) {
        return {
            closeness: 0,
            error: "data must be greater than pattern length",
        };
    }
    if (ohlc_data.length < 10) {
        return {
            closeness: 0,
            error: "data must be greater than or equals to 10",
        };
    }
    const src_pattern = match_pattern_features(ohlc_pattern);
    const dst_data = match_pattern_features(ohlc_data);
    const dst_pattern = [];
    // Calculate Slicing from Data and Pattern
    for (const [idx, tick] of ohlc_pattern.entries()) {
        const tick_percentage = round(tick.tick_distance, 2);
        const data_idx = round(tick_percentage * 0.01 * (ohlc_data.length - 1));
        dst_pattern.push(dst_data[data_idx]);
    }
    // Calculate Pattern Closeness
    let closeness = 0;
    for (const [idx, tick] of src_pattern.entries()) {
        const src_tick = src_pattern[idx];
        const dst_tick = dst_pattern[idx];
        let ct = 0;
        let current_closeness = 0;
        if (settings.upper_wick_percent) {
            ct += 1;
            current_closeness += calculate_closeness_gradient(src_tick, dst_tick, "upper_wick_percent");
        }
        if (settings.lower_wick_percent) {
            ct += 1;
            current_closeness += calculate_closeness_gradient(src_tick, dst_tick, "lower_wick_percent");
        }
        if (settings.mid_diff_percent) {
            ct += 1;
            current_closeness += calculate_closeness_gradient(src_tick, dst_tick, "mid_diff_percent");
        }
        if (settings.close_diff_percent) {
            ct += 1;
            current_closeness += calculate_closeness_gradient(src_tick, dst_tick, "close_diff_percent");
        }
        if (settings.volume_diff_percent) {
            ct += 1;
            current_closeness += calculate_closeness_gradient(src_tick, dst_tick, "volume_diff_percent");
        }
        current_closeness = ct ? current_closeness * (4 / ct) : 0;
        closeness += current_closeness;
    }
    return {
        closeness: round(closeness / src_pattern.length, 2),
        error: "",
    };
};
const calculate_closeness_gradient = (source, target, key) => {
    let current_closeness = 0;
    if (source[key] >= 0.99 * target[key] && source[key] <= 1.01 * target[key]) {
        current_closeness = 25;
    }
    else if (source[key] >= 0.98 * target[key] && source[key] <= 1.02 * target[key]) {
        current_closeness = 24;
    }
    else if (source[key] >= 0.96 * target[key] && source[key] <= 1.04 * target[key]) {
        current_closeness = 23;
    }
    else if (source[key] >= 0.94 * target[key] && source[key] <= 1.06 * target[key]) {
        current_closeness = 21;
    }
    else if (source[key] >= 0.92 * target[key] && source[key] <= 1.08 * target[key]) {
        current_closeness = 19;
    }
    else if (source[key] >= 0.9 * target[key] && source[key] <= 1.1 * target[key]) {
        current_closeness = 17;
    }
    else if (source[key] >= 0.8 * target[key] && source[key] <= 1.2 * target[key]) {
        current_closeness = 15;
    }
    else if (source[key] >= 0.7 * target[key] && source[key] <= 1.3 * target[key]) {
        current_closeness = 13;
    }
    else if (source[key] >= 0.6 * target[key] && source[key] <= 1.4 * target[key]) {
        current_closeness = 11;
    }
    else if (source[key] >= 0.5 * target[key] && source[key] <= 1.5 * target[key]) {
        current_closeness = 9;
    }
    else if (source[key] >= 0.4 * target[key] && source[key] <= 1.6 * target[key]) {
        current_closeness = 7;
    }
    else if (source[key] >= 0.3 * target[key] && source[key] <= 1.7 * target[key]) {
        current_closeness = 5;
    }
    else if (source[key] >= 0.2 * target[key] && source[key] <= 1.8 * target[key]) {
        current_closeness = 3;
    }
    else if (source[key] >= 0.1 * target[key] && source[key] <= 1.9 * target[key]) {
        current_closeness = 1;
    }
    return current_closeness;
};
//# sourceMappingURL=feature_pattern_matcher.js.map